#include <DS3231.h>
#include <Wire.h>
#include <LedControl.h>
#include "symbols.h"
#include "button.h"

const byte PANEL_COUNT = 4;

DS3231 rtc;
//DIN, CLK, CS
LedControl lc(1, 5, 4, PANEL_COUNT);  
BUTTON btn(3);

const byte HH_MAX = 24;
const byte MM_MAX = 60;

bool h12 = false;
bool PM;

byte hh, mm, prev;

enum {rsNormal, rsSetH, rsSetM} rtcState;

void showTime(void) {
  byte f = 0;
  byte t = PANEL_COUNT;

  switch (rtcState) {
    case rsSetH: 
      f = PANEL_COUNT >> 1;
      for (byte i=0; i<f; i++) lc.clearDisplay(i);
      break;
    case rsSetM: 
      t = PANEL_COUNT >> 1;  
      for (byte i=t; i<PANEL_COUNT; i++) lc.clearDisplay(i);
      break;
  }
  
  byte digit;
  byte shift;
  
  for (byte i=f; i<t; i++) {
    switch (i) {
      case 0: digit = mm%10; shift = 1; break;
      case 1: digit = mm/10; shift = 0; break;
      case 2: digit = hh%10; shift = 3; break;
      case 3: digit = hh/10; shift = 2; break;
    }
   
    for (byte j=0; j<ROW_COUNT; j++) { 
      lc.setRow(i, j, pgm_read_byte(&symbols[digit][j]) << shift);
    }
  }

  //Двоеточие
  lc.setLed(1, 1, 0, true);
  lc.setLed(1, 2, 0, true);
  lc.setLed(1, 5, 0, true);
  lc.setLed(1, 6, 0, true);

  lc.setLed(2, 1, 7, true);
  lc.setLed(2, 2, 7, true);
  lc.setLed(2, 5, 7, true);
  lc.setLed(2, 6, 7, true);
}

void setup() {
  Wire.begin();
  
  for (byte i=0; i<PANEL_COUNT; i++) {
    lc.shutdown(i, false);
    lc.setIntensity(i, 8);
    lc.clearDisplay(i);
  }
}

void loop() {
  //Нормальная работа
  if (rtcState == rsNormal) {
    mm = rtc.getMinute();

    if (mm != prev) {
      hh = rtc.getHour(h12, PM);
      showTime();
      prev = mm;
    }
  }
  
  //Нажатие на кнопку
  switch (btn.isChanged()) {
    //Короткое нажатие
    case bsPressed:
      switch (rtcState) {
        case rsSetH: hh = ++hh % HH_MAX; break;
        case rsSetM: mm = ++mm % MM_MAX; break;  
      }
      showTime(); 
      break;

    //Длинное нажатие  
    case bsLongPressed:
      switch (rtcState) {
        case rsNormal: rtcState = rsSetH; break;
        case rsSetH: rtc.setHour(hh); rtcState = rsSetM; break;
        case rsSetM: rtc.setMinute(mm); rtcState = rsNormal; break;
      }
      
      showTime();
      break;  
  }
}
